/*
 REXX program to extract statistics from NFTP.FLS file     Version 0.7
 Freely distributable, but copyright by Tth Ferenc <etus@alarmix.net>
 Please mail any suggestinos and/or bug report to the author.

 To do (if I have time :)
 - option to count uploads/downloads separately
 - option to sort sites according to speed or down/upload size
 - option to list only specified time interval or sites

 History

  0.7
    - stat.cmd was failing if log file contained file sizes over 100MB
    - small modifications such that the program will now run under
      Windows NT (tested under Regina REXX)
    - cleaned up source code a little bit
    - added REXX sort routine, no need to call sort.exe anymore
    - improved performance
  0.6
    - corrected a small bug in the routine that handles temporary files
    - added progress indicator so the user won't worry that much :)
    - much cleaner source code with more comments
  0.5
    - corrected a bug, now the first site listed in nftp.fls will be listed


   Variables used for calculation
   ------------------------------
   files:  number of entries in NFTP log files
   month.: month of entry
   year.:  year of entry
   size.:  filesize of entry
   [more to-be added here if I have the time]
   
*/

/* Introduce ourselves :) */
Say
Say "NFTP logfile statistics calculator v0.7"
Say

/* Pull in the log file names  */
Parse Arg inputfiles

/* Temporary filename */
tempfile = "nftpstat.tmp"

/* Output filename */
outputfile = "nftp.sta"

/* Delete output file */
"@del "||outputfile||" 1> nul 2>&1"

/* Initialize variables */
noecho = call SysCurState 'OFF'
ProgressCounter=1
process="|/-\"

/* Check wheter inputfiles were given or not on the command line */
If inputfiles="" Then
Do
  Say "usage: stat.cmd NFTP_log_filename"
  Say "where NFTP_log_filename is a filemask describing NFTP log files"
  Exit
End

/* get all log files into one big temporary file */
"@copy "||inputfiles||" "||tempfile||" 1> nul 2>&1"

/* open our outputfile */
noecho = Lineout(outputfile, "Logfile created by NFTP logfile stats calculator v0.7 on "||DATE())
noecho = Lineout(outputfile, "")
noecho = Lineout(outputfile, "Date        Megabytes    Average speed (cps)")
noecho = Lineout(outputfile, "--------------------------------------------")

/* Inform our user what is happening */
noecho = Charout(, "Gathering information from logfiles...")

/* Calculate the number of files and store their date/time, size etc. */
files=1

Do While Lines(tempfile) > 0
  Call PROGRESSINDICATOR(files)
  Entry = Linein(tempfile)

/* Check if the entry is old style or new style.
   Old style log entries do not log the speed */
  If Pos("B/s) ", entry) = 0 Then
  Do
    Parse var entry '/' month.files '/' year.files .':'. size.files . site.files ':' .
    size.files = Strip(size.files)
    speed.files = -1
  End
  Else
  Do
    Parse var entry '/' month.files '/' year.files .':'. size.files '(' speed.files 'B/s)'. site.files ':' .
    speed.files = Strip(speed.files)
    size.files = Strip(size.files)
  End
    /* new style with speed */

/* If bps = 0 then let it be equal with filesize since in most cases
 it means that the transfer was done in 1 sec. But don't forget that
 there can be 0 long files */

  If speed.files = 0 Then speed.files = size.files
  If speed.files = 0 Then speed.files = 1
  site = site.files
  bytespersite.site=0
  bytespersite2.site=0
  offset=year.files*10000+month.files
  bytespermonth.offset=0
  bytespermonth2.offset=0
  timepersite.site=0
  timepermonth.offset=0
  files=files+1
End
call Lineout tempfile
files=files-1
time=0
total=0
total2=0
totaltime=0
say " "

noecho = Charout(, "Calculating statistics...")

/* summary */
Do counter=1 To files
/* Calculate the number of files and store their date/time, size etc. */
  Call PROGRESSINDICATOR(counter)
  site = site.counter
  sps = size.counter / speed.counter
  bytespersite.site=bytespersite.site + size.counter
  offset = year.counter * 10000 + month.counter
  bytespermonth.offset=bytespermonth.offset+size.counter
  If speed.counter <> -1 Then
  Do
    bytespersite2.site=bytespersite2.site + size.counter
    timepersite.site=timepersite.site+sps
    bytespermonth2.offset = bytespermonth2.offset + size.counter
    timepermonth.offset = timepermonth.offset + sps
    total2 = total2 + size.counter
    total = total + size.counter
    totaltime = totaltime + sps
  End
  Else total2=total2+size.counter
End

kiirt=0
Do numsites=1 To files
  Call PROGRESSINDICATOR(numsites)
  offset=year.numsites*10000+month.numsites
  If numsites > 1 Then
  Do
    check = kiirt
    Do While (offset <> honap.check) & (check > 0)
      check = check - 1
    End
  End

  If (offset <> honap.check) Then
  Do
    If timepermonth.offset=0 Then timepermonth.offset=1
    speed=Right(THOUSANDSEPARATOR(Format(bytespermonth2.offset/timepermonth.offset,,0)),16)
    if speed=0 Then speed=""
    byte=Format(bytespermonth.offset/1048576,11,2)
    index = kiirt + 1
    sums.index = year.numsites"/"month.numsites||byte||speed
    honap.kiirt = offset
    kiirt=kiirt+1
  End
End
sums.0 = kiirt;
call qqsort 1, sums.0
Do i = 1 to sums.0
  noecho = Lineout(outputfile, sums.i)
End

noecho = Lineout(outputfile, "")
noecho = Lineout(outputfile, "**************************************************************************")
noecho = Lineout(outputfile, "")
noecho = Lineout(outputfile, "Site                                      Megabytes    Average speed (cps)")
noecho = Lineout(outputfile, "--------------------------------------------------------------------------")

Do numsites=1 To files
  Call PROGRESSINDICATOR(numsites)
  site=site.numsites
  check=0
  If numsites > 1 Then
    Do
      check = numsites -1
      Do While (site.check <> site) & (check > 0)
        check = check - 1
      End
    End

  If (site.check<>site) Then Do
    If timepersite.site=0 Then timepersite.site=1
    speed=THOUSANDSEPARATOR(Format(bytespersite2.site/timepersite.site,,0))
    speed=Right(speed,16)
    byte=Format(bytespersite.site/1048576,9,2)
    If speed=0 Then speed=""
    donotecho = Lineout(outputfile, Overlay(" ", site,38,0)||byte||"    "||speed)
  End
End
say " "
If totaltime>0 Then totalspeed=Format(total/totaltime,,0)
Else totalspeed = ""
noecho = Lineout(outputfile, "--------------------------------------------------------------------------")
noecho = Lineout(outputfile, "Total            " Right(THOUSANDSEPARATOR(Format(total2/1048576,,2)), 31) Right(THOUSANDSEPARATOR(totalspeed), 19))
say "Done calculating statistics: 'nftp.sta'."
"@del "||tempfile||" 1> nul 2> nul"
Exit

/* Show progress indicator */
PROGRESSINDICATOR:
Parse Arg Variable
If Variable // 100 = 0 Then
Do
  noecho = Charout(, SubStr(Process, 1 + ProgressCounter // 5, 1)||d2c(8))
  ProgressCounter = ProgressCounter + 1
End
Return

/* Insert thousand separators */
THOUSANDSEPARATOR:
Parse Arg Variable
Decimal = ""
If Pos(".", Variable) > 0 Then 
Do
  Decimal = SubStr(Variable, Pos(".", Variable))
  Variable = SubStr(Variable, 1, Pos(".", Variable) - 1)
End
Len = (Length(Variable) - 1) / 3
Result = Reverse(Variable)
Do i = 0 To Len - 1
  Result = Insert(',', Result, 3 + i * 4)
End
Result = Reverse(Result)||Decimal
Return Result



/* ------------------------------------------------------------------ */
/* function: quick sort routine                                       */
/*                                                                    */
/* call:     QuickSort first_element, last_element                    */
/*                                                                    */
/* returns:  nothing                                                  */
/*                                                                    */
/* notes:    You must save the elements to sort in the stem "sums."   */
/*           sums.0 must contain the number of elements in the stem.  */
/*                                                                    */
/* Author: Ruediger Wilke                                             */
/*                                                                    */
qqsort: procedure expose sums.
 
   arg lf, re
 
   if re -lf < 9 then
     do lf = lf to re -1
 
       m = lf
 
       do j = lf +1 to re
         if sums.j < sums.m then                                   /* v2.80 */
           m = j
       end /* j = lf +1 to re */
 
       t = sums.m; sums.m = sums.lf; sums.lf = t
 
     end /* lf = lf to re -1 */
     else
     do
       i = lf
       j = re
       k = (lf + re)%2
       t = sums.k
 
       do until i > j
 
         do while sums.i < t                                    /* v2.80 */
           i = i + 1
         end /* while sums.i << t */
 
         do while sums.j > t                                    /* v2.80 */
           j = j - 1
         end /* while sums.j >> t */
 
         if i <= j then
         do
           xchg = sums.i
           sums.i = sums.j
           sums.j = xchg
           i = i + 1
           j = j - 1
         end /* if i <= j then */
 
       end /* until i > j */
 
       call qqsort lf, j
       call qqsort i, re
     end /* else */
 
 return
